@@ -13,7 +13,7 @@ import time
 import traceback
 from datetime import datetime
 from multiprocessing import Process
-from typing import Callable, Dict, List, Optional, Set, Tuple
+from typing import Callable, cast, Dict, List, Optional, Set, Tuple
 from urllib.parse import urlsplit
 
 import yaml
@@ -25,9 +25,12 @@ from zuper_commons.text import box, indent
 from zuper_commons.timing import now_utc
 from zuper_commons.types import ZException
 
+from dt_shell import ConfigNotPresent
+from dt_shell.config import get_shell_config_default, read_shell_config
 from duckietown_build_utils import (
     docker_pull_retry,
     DockerCompleteImageName,
+    DockerCredentials,
     get_complete_tag,
     parse_complete_tag,
 )
@@ -60,6 +63,7 @@ from duckietown_challenges import (
     StepName,
     SubmissionID,
     WorkSubmissionResultDict,
+    ZValueError,
 )
 from duckietown_docker_utils.docker_run import get_developer_volumes
 from . import __version__, logger
@@ -79,9 +83,7 @@ from .exceptions import UserError
 from .ipfs_utils import ipfs_available, IPFSException
 from .logging_capture import setup_logging
 from .logging_worker import write_logs_worker
-from .login import do_docker_login
 from .misc import list_all_files
-from .shell_config import DTShellConstants, read_shell_config
 from .types import ContainerName, ServiceName
 from .uploading import download_artefacts, get_files_to_upload, try_s3, upload, upload_files
 
@@ -90,27 +92,6 @@ def is_solution_container(container_name: str) -> bool:
     return "solution" in container_name
 
 
-def get_token_from_shell_config():
-    k = DTShellConstants.DT1_TOKEN_CONFIG_KEY
-
-    if k in os.environ:
-        # msg = 'Using token passed in environment variable %s' % k
-        # logger.debug(msg)
-        return os.environ[k]
-    else:
-        pass
-
-        # msg = 'Could not find the env variable %s; reading shell config.' % k
-        # logger.debug(msg)
-
-    shell_config = read_shell_config()
-    if shell_config.token_dt1 is None:
-        msg = "Please set a Duckietown Token using the command `dts tok set`."
-        raise UserError(msg)
-    else:
-        return shell_config.token_dt1
-
-
 def dt_challenges_evaluator():
     # noinspection PyBroadException
     try:
@@ -124,7 +105,11 @@ def dt_challenges_evaluator():
 
 
 # noinspection PyBroadException
-def dt_challenges_evaluator_(args: List[str] = None, token: str = None):
+def dt_challenges_evaluator_(
+    args: List[str] = None, token: Optional[str] = None, credentials: Optional[DockerCredentials] = None
+):
+    if credentials is None:
+        credentials = {}
     # from duckietown_challenges.col_logging import setup_logging
     #
     # setup_logging()
@@ -136,28 +121,95 @@ Usage:
 
 """
     prog = "dt-challenges-evaluator"
-    parser = argparse.ArgumentParser(prog=prog, usage=usage)
-    parser.add_argument("--version", action="store_true", default=False)
-    parser.add_argument("--continuous", action="store_true", default=False)
-    parser.add_argument("--no-pull", dest="no_pull", action="store_true", default=False)
-    parser.add_argument("--no-upload", dest="no_upload", action="store_true", default=False)
-    parser.add_argument("--no-delete", dest="no_delete", action="store_true", default=False)
-    parser.add_argument("--no-cache", dest="no_cache", action="store_true", default=False)
+
+    # noinspection PyTypeChecker
+    parser = argparse.ArgumentParser(
+        prog=prog, usage=usage, formatter_class=argparse.ArgumentDefaultsHelpFormatter
+    )
+
+    def true_if_present(v: str):
+        v2 = v.upper().replace("-", "_")
+        ev = f"DT_RUNNER_{v2}"
+        res = ev in os.environ
+        if not res:
+            logger.debug(f"not found {ev}")
+        return res
+
+    def get_env_value_or_none(v: str, T, default_value=None):
+        v2 = v.upper().replace("-", "_")
+        ev = f"DT_RUNNER_{v2}"
+        if ev not in os.environ:
+            logger.debug(f"not found {ev}")
+            return default_value
+        val = os.environ[ev]
+        res = T(val)
+        return res
+
+    parser.add_argument("--version", action="store_true", default=False, help="Shows version and exits")
+    parser.add_argument(
+        "--continuous", action="store_true", default=true_if_present("continuous"), help="Runs continuously"
+    )
+    parser.add_argument(
+        "--no-pull",
+        dest="no_pull",
+        action="store_true",
+        default=true_if_present("no-pull"),
+        help="Avoid pulling images",
+    )
+    parser.add_argument(
+        "--no-upload",
+        dest="no_upload",
+        action="store_true",
+        default=true_if_present("no-upload"),
+        help="Avoid uploading artefacts",
+    )
+    parser.add_argument(
+        "--no-delete",
+        dest="no_delete",
+        action="store_true",
+        default=true_if_present("no-delete"),
+        help="Avoids deleting local evaluation",
+    )
+    parser.add_argument(
+        "--no-cache",
+        dest="no_cache",
+        action="store_true",
+        default=true_if_present("no-cache"),
+        help="Avoiding using local cache",
+    )
     parser.add_argument("--ipfs", dest="ipfs", action="store_true", default=False)
     parser.add_argument("--one", dest="one", action="store_true", default=False)
     parser.add_argument("--keep-registry", dest="do_not_mess_with_repo", action="store_true", default=False)
-    parser.add_argument("--machine-id", default=None, help="Machine name")
-    parser.add_argument("--name", default=None, help="Evaluator name")
-    parser.add_argument("--impersonate", default=None)
-    parser.add_argument("--quota-cpu", type=float, default=None, help="average number of CPUs")
+    parser.add_argument(
+        "--machine-id", help="Machine name", default=get_env_value_or_none("machine-id", str),
+    )
+    parser.add_argument("--name", help="Evaluator name", default=get_env_value_or_none("name", str))
+    parser.add_argument(
+        "--impersonate", help="User ID to impersonate", default=get_env_value_or_none("impersonate", str)
+    )
+    parser.add_argument(
+        "--quota-cpu",
+        type=float,
+        help="average number of CPUs",
+        default=get_env_value_or_none("quota-cpu", float),
+    )
     DEFAULT_TMPDIR = "/tmp/duckietown/DT18/evaluator/executions"
 
-    parser.add_argument("--tmpdir", default=DEFAULT_TMPDIR)
+    parser.add_argument(
+        "--tmpdir", help="Location of temp dir", default=get_env_value_or_none("tmpdir", str, DEFAULT_TMPDIR)
+    )
+    parser.add_argument(
+        "--credentials",
+        help="Credentials YAML dictionary",
+        default=get_env_value_or_none("credentials", str, None),
+    )
     parser.add_argument("--submission", default=None, help="evaluate this particular submission")
     parser.add_argument(
         "--reset", dest="reset", action="store_true", default=False, help="Reset submission",
     )
-    parser.add_argument("--features", default="{}")
+    parser.add_argument(
+        "--features", help="Features to declare", default=get_env_value_or_none("features", str, "{}")
+    )
     parser.add_argument("--debug-volumes", default=None)
     parsed = parser.parse_args(args=args)
     if parsed.submission and parsed.continuous:
@@ -172,6 +224,17 @@ Usage:
         return
 
     copy_to_machine_cache = not parsed.no_cache
+
+    if parsed.credentials:
+        c = parsed.credentials
+        try:
+            creds = yaml.load(c, Loader=yaml.Loader)
+        except Exception:
+            msg = "Cannot read credentials passed in env/cmd line"
+            raise ZValueError(msg, c=c)
+        logger.info("Read credenitals", environment=list(creds), direct=list(credentials))
+        credentials.update(creds)
+
     check_docker_environment()
     try:
         check_executable_exists("docker-compose")
@@ -211,8 +274,16 @@ Usage:
 
     allow_host_network = False
     quota_cpu = parsed.quota_cpu
+    try:
+        shell_config = read_shell_config()
+    except ConfigNotPresent as e:
+        msg = f"Cannot find shell config: {e}"
+        logger.warning(msg)
+        shell_config = get_shell_config_default()
+
+    credentials.update(cast(DockerCredentials, shell_config.docker_credentials))
     if token is None:
-        token = get_token_from_shell_config()
+        token = shell_config.token_dt1
 
     if parsed.continuous:
 
@@ -242,6 +313,7 @@ Usage:
                     use_ipfs=parsed.ipfs,
                     do_not_mess_with_repo=parsed.do_not_mess_with_repo,
                     quota_cpu=quota_cpu,
+                    credentials=credentials,
                 )
                 multiplier = 1.0
                 if parsed.one:
@@ -302,6 +374,7 @@ Usage:
                     use_ipfs=parsed.ipfs,
                     do_not_mess_with_repo=parsed.do_not_mess_with_repo,
                     quota_cpu=quota_cpu,
+                    credentials=credentials,
                 )
             except NothingLeft as e:
                 if submission_id is None:
@@ -453,7 +526,8 @@ def upload_process(
                 uploaded=_uploaded,
                 features={},
                 # features=features,
-                query_string=f"phase=upload_process&job_id={job_id}&version={evaluator_version}&machine_id={machine_id}",
+                query_string=f"phase=upload_process&job_id={job_id}&version="
+                f"{evaluator_version}&machine_id={machine_id}",
             )
         except:
             logger.warning(traceback.format_exc())
@@ -478,6 +552,7 @@ def go_(
     tmpdir: str,
     copy_to_machine_cache: bool,
     quota_cpu: Optional[float],
+    credentials: DockerCredentials,
     token: str = None,
     impersonate: Optional[int] = None,
     debug_volumes: Optional[str] = None,
@@ -492,8 +567,14 @@ def go_(
     #     logger.info(f"Waiting because free CPU is {features['processor_free_percent']}%.")
     #     return
 
+    try:
+        shell_config = read_shell_config()
+    except (ConfigNotPresent) as e:
+        msg = f"Cannot find shell config: {e}"
+        logger.warning(msg)
+        shell_config = get_shell_config_default()
     if token is None:
-        token = get_token_from_shell_config()
+        token = shell_config.token_dt1
 
     url = get_duckietown_server_url()
 
@@ -536,13 +617,6 @@ def go_(
     job_id = res["job_id"]
     aws_config: Optional[AWSConfig] = res.get("aws_config", None)
 
-    docker_username = os.environ.get("DOCKER_USERNAME", None)
-    docker_password = os.environ.get("DOCKER_PASSWORD", None)
-
-    if docker_password and docker_username:
-        do_docker_login(docker_username, docker_password)
-    else:
-        logger.warning("Docker credentials not found in environment")
     #
     # handler = TermHandler(
     #     token=token,
@@ -577,6 +651,7 @@ def go_(
 
         # noinspection PyBroadException
         try:
+            query_string = f"phase=main&job_id={job_id}&version={evaluator_version}&machine_id={machine_id}"
             res_ = dtserver_job_heartbeat(
                 token,
                 job_id=job_id,
@@ -586,7 +661,7 @@ def go_(
                 impersonate=impersonate,
                 uploaded=[],
                 features={},
-                query_string=f"phase=main&job_id={job_id}&version={evaluator_version}&machine_id={machine_id}",
+                query_string=query_string,
             )
 
             abort = res_["abort"]
@@ -611,10 +686,10 @@ def go_(
         copy_to_machine_cache=copy_to_machine_cache,
         allow_host_network=allow_host_network,
         use_ipfs=use_ipfs,
-        server_host=server_host,
         cd=cd,
         heartbeat=heartbeat,
         quota_cpu=quota_cpu,
+        credentials=credentials,
     )
 
     msg = "Reporting status %s for job %s submission %s.\n\n%s\n\n%s\n\n%s" % (
@@ -672,10 +747,10 @@ def get_cr(
     copy_to_machine_cache: bool,
     allow_host_network: bool,
     use_ipfs: bool,
-    server_host: str,
     quota_cpu: Optional[float],
     cd: Optional[ChallengeDescription],
-    heartbeat: Optional[Callable] = None,
+    heartbeat: Optional[Callable],
+    credentials: DockerCredentials,
 ) -> Tuple[ChallengeResults, List]:
     aws_config = res["aws_config"]
     if aws_config and do_upload:
@@ -760,7 +835,7 @@ def get_cr(
                     timeout_sec=timeout_sec,
                     allow_host_network=allow_host_network,
                     use_ipfs=use_ipfs,
-                    server_host=server_host,
+                    # server_host=server_host,
                     require_scores=require_scores,
                     heartbeat=heartbeat,
                     quota_cpu=quota_cpu,
@@ -768,6 +843,7 @@ def get_cr(
                     submitter_name=submitter_name,
                     steps2scores=steps2scores,
                     evaluator_name=evaluator_name,
+                    credentials=credentials,
                 )
         finally:
             if do_upload:
@@ -835,8 +911,9 @@ def run_single(
     submission_id: SubmissionID,
     project: str,
     do_pull: bool,
-    server_host: Optional[str],
+    # server_host: Optional[str],
     evaluator_name: str,
+    credentials: DockerCredentials,
     quiet: bool = False,
     debug_volumes: Optional[str] = None,
     timeout_sec: float,
@@ -848,6 +925,7 @@ def run_single(
 ):
     """
 
+    :param credentials:
     :param evaluator_name:
     :param submitter_name:
     :param submission_id:
@@ -862,7 +940,6 @@ def run_single(
     :param challenge_step_name:
     :param project:
     :param do_pull:
-    :param server_host:
     :param debug_volumes:
     :param timeout_sec:
     :param quota_cpu:
@@ -903,7 +980,7 @@ def run_single(
     config = get_config(
         wd=wd,
         fd=fd,
-        server_host=server_host,
+        # server_host=server_host,
         challenge_parameters_=challenge_parameters,
         solution_container=solution_container,
         debug_volumes=debug_volumes,
@@ -944,7 +1021,7 @@ def run_single(
             image = s["image"]
             if "dummy" in image:
                 continue
-            docker_pull_retry(client, image, ntimes=4, wait=5)
+            docker_pull_retry(client, image, ntimes=4, wait=5, credentials=credentials, quiet=False)
     if valid_config:
         services_names = list(challenge_parameters.services)
         # print('services: %s' % services_names)
@@ -958,8 +1035,10 @@ def run_single(
             quota_cpu=quota_cpu,
             quiet=quiet,
         )
-
-        write_logs(wd, project, services=config["services"])
+        try:
+            write_logs(wd, project, services=config["services"])
+        except DockerComposeFail:
+            logger.error("Cannot write logs", tb=traceback.format_exc())
     else:
         status = ChallengesConstants.STATUS_JOB_ERROR
 
@@ -1023,7 +1102,7 @@ def run_one(
     quota_cpu: Optional[float],
     heartbeat: Optional[Callable] = None,
     quiet: bool = False,
-):
+) -> ChallengeResults:
     """
 
     :param wd:
@@ -1056,10 +1135,11 @@ def run_one(
         try:
             t0 = time.time()
 
-            client = check_docker_environment()
+            time.sleep(10)
             service2id = get_services_id(wd, project, services)
             # logger.debug(service2id=service2id)
-            enable_quotas = False
+            client = check_docker_environment()
+            enable_quotas = True
             if enable_quotas:
                 for service_name, container_id in service2id.items():
                     container: Container = client.containers.get(container_id)
@@ -1316,7 +1396,7 @@ def get_config(
     challenge_parameters_: EvaluationParameters,
     solution_container: DockerCompleteImageName,
     add_extra_environment: Dict[str, str],
-    server_host: Optional[str],
+    # server_host: Optional[str],
     labels: Dict[str, str],
     debug_volumes: Optional[str] = None,
     allow_host_network: bool = False,
